@@ -3,7 +3,7 @@ set -ex
 
 
 #---
-#   MinGW cross compiling environment  (1.1)
+#   MinGW cross compiling environment  (1.2)
 #   =================================
 #
 #   http://www.profv.de/mingw_cross_env/
@@ -100,46 +100,47 @@ SOURCEFORGE_MIRROR=downloads.sourceforge.net
 
 PATH="$PREFIX/bin:$PATH"
 
-VERSION_mingw_runtime=3.12
-VERSION_w32api=3.9
-VERSION_binutils=2.17.50-20060824-1
-VERSION_gcc=3.4.5-20060117-1
-VERSION_pkg_config=0.21
+VERSION_mingw_runtime=3.13
+VERSION_w32api=3.10
+VERSION_binutils=2.18.50-20071123
+VERSION_gcc=4.2.1-2
+VERSION_pkg_config=0.22
 VERSION_pthreads=2-8-0
 VERSION_zlib=1.2.3
 VERSION_pdcurses=33
-VERSION_gettext=0.16.1
+VERSION_gettext=0.17
 VERSION_libiconv=1.9.2
-VERSION_winpcap=4_0_1
+VERSION_tre=0.7.5
+VERSION_winpcap=4_0_2
 VERSION_libdnet=1.11
-VERSION_libgpg_error=1.5
-VERSION_libgcrypt=1.2.4
+VERSION_libgpg_error=1.6
+VERSION_libgcrypt=1.4.0
 VERSION_gnutls=1.6.3
-VERSION_libxml2=2.6.29
-VERSION_libxslt=1.1.21
+VERSION_libxml2=2.6.30
+VERSION_libxslt=1.1.22
 VERSION_xmlwrapp=0.5.0
-VERSION_curl=7.16.4
-VERSION_libpng=1.2.18
+VERSION_curl=7.17.1
+VERSION_libpng=1.2.23
 VERSION_jpeg=6b
 VERSION_tiff=3.8.2
-VERSION_giflib=4.1.4
+VERSION_giflib=4.1.6
 VERSION_freetype=2.3.5
 VERSION_fontconfig=2.4.2
 VERSION_libmikmod=3.2.0-beta2
 VERSION_ogg=1.1.3
-VERSION_vorbis=1.1.2
+VERSION_vorbis=1.2.0
 VERSION_gd=2.0.35
 VERSION_SDL=1.2.11
 VERSION_smpeg=0.4.5+cvs20030824
-VERSION_SDL_mixer=1.2.7
-VERSION_SDL_image=1.2.5
+VERSION_SDL_mixer=1.2.8
+VERSION_SDL_image=1.2.6
 VERSION_fltk=1.1.7
-VERSION_geos=3.0.0rc4
+VERSION_geos=3.0.0rc5
 VERSION_proj=4.5.0
-VERSION_libgeotiff=1.2.3
+VERSION_libgeotiff=1.2.4
 VERSION_gdal=1.4.2
-VERSION_pdflib_lite=7.0.1
-VERSION_libowfat=0.25
+VERSION_pdflib_lite=7.0.2
+VERSION_libowfat=0.26
 
 
 #---
@@ -317,9 +318,10 @@ case "$1" in
     ;;
 
 --build)
+    VERSION_DIRNAME_binutils=`echo "$VERSION_binutils" | sed 's,-.*,,'`
     cd "$SOURCE"
     tar xfvz "$DOWNLOAD/binutils-$VERSION_binutils-src.tar.gz"
-    cd "binutils-$VERSION_binutils-src"
+    cd "binutils-$VERSION_DIRNAME_binutils"
     ./configure \
         --target="$TARGET" \
         --prefix="$PREFIX" \
@@ -330,7 +332,7 @@ case "$1" in
         --disable-shared
     $MAKE all install
     cd "$SOURCE"
-    rm -rfv "binutils-$VERSION_binutils-src"
+    rm -rfv "binutils-$VERSION_DIRNAME_binutils"
     ;;
 
 esac
@@ -347,7 +349,7 @@ case "$1" in
 --new-versions)
     VERSION=`
         wget -q -O- 'http://sourceforge.net/project/showfiles.php?group_id=2435' |
-        $SED -n 's,.*gcc-core-\([0-9][^>]*\)-src\.tar.*,\1,p' | 
+        $SED -n 's,.*gcc-\([0-9][^>]*\)-src\.tar.*,\1,p' | 
         head -1`
     test -n "$VERSION"
     $SED "s,^VERSION_gcc=.*,VERSION_gcc=$VERSION," -i "$0"
@@ -355,17 +357,14 @@ case "$1" in
 
 --download)
     cd "$DOWNLOAD"
-    tar tfz "gcc-core-$VERSION_gcc-src.tar.gz" &>/dev/null ||
-    wget -c "http://$SOURCEFORGE_MIRROR/mingw/gcc-core-$VERSION_gcc-src.tar.gz"
-    tar tfz "gcc-g++-$VERSION_gcc-src.tar.gz" &>/dev/null ||
-    wget -c "http://$SOURCEFORGE_MIRROR/mingw/gcc-g++-$VERSION_gcc-src.tar.gz"
+    tar tfz "gcc-$VERSION_gcc-src.tar.gz" &>/dev/null ||
+    wget -c "http://$SOURCEFORGE_MIRROR/mingw/gcc-$VERSION_gcc-src.tar.gz"
     ;;
 
 --build)
     cd "$SOURCE"
-    tar xfvz "$DOWNLOAD/gcc-core-$VERSION_gcc-src.tar.gz"
-    tar xfvz "$DOWNLOAD/gcc-g++-$VERSION_gcc-src.tar.gz"
-    cd "gcc-$VERSION_gcc"
+    tar xfvz "$DOWNLOAD/gcc-$VERSION_gcc-src.tar.gz"
+    cd "gcc-$VERSION_gcc-src"
     ./configure \
         --target="$TARGET" \
         --prefix="$PREFIX" \
@@ -382,7 +381,7 @@ case "$1" in
         --enable-sjlj-exceptions
     $MAKE all install
     cd "$SOURCE"
-    rm -rfv "gcc-$VERSION_gcc"
+    rm -rfv "gcc-$VERSION_gcc-src"
     ;;
 
 esac
@@ -634,6 +633,46 @@ case "$1" in
 esac
 
 
+#---
+#   TRE
+#
+#   http://laurikari.net/tre/
+#---
+
+case "$1" in
+
+--new-versions)
+    VERSION=`
+        wget -q -O- 'http://laurikari.net/tre/download.html' |
+        $SED -n 's,.*tre-\([a-z0-9][^>]*\)\.tar.*,\1,p' |
+        head -1`
+    test -n "$VERSION"
+    $SED "s,^VERSION_tre=.*,VERSION_tre=$VERSION," -i "$0"
+    ;;
+
+--download)
+    cd "$DOWNLOAD"
+    tar tfj "tre-$VERSION_tre.tar.bz2" &>/dev/null ||
+    wget -c "http://laurikari.net/tre/tre-$VERSION_tre.tar.bz2"
+    ;;
+
+--build)
+    cd "$SOURCE"
+    tar xfvj "$DOWNLOAD/tre-$VERSION_tre.tar.bz2"
+    cd "tre-$VERSION_tre"
+    ./configure \
+        --host="$TARGET" \
+        --disable-shared \
+        --prefix="$PREFIX/$TARGET" \
+        --disable-nls
+    $MAKE install bin_PROGRAMS= sbin_PROGRAMS= noinst_PROGRAMS=
+    cd "$SOURCE"
+    rm -rfv "tre-$VERSION_tre"
+    ;;
+
+esac
+
+
 #---
 #   WinPcap
 #
@@ -668,6 +707,7 @@ case "$1" in
     cp -p common/Ntddpack.h   common/ntddpack.h
     cp -p common/Packet32.h   common/packet32.h
     cp -p common/WpcapNames.h common/wpcapnames.h
+    $SED 's,(PCHAR)winpcap_hdr +=,winpcap_hdr +=,' -i Packet9x/DLL/Packet32.c
     $TARGET-gcc -Icommon -O -c Packet9x/DLL/Packet32.c
     $TARGET-ar rc libpacket.a Packet32.o
     $TARGET-ranlib libpacket.a
@@ -676,7 +716,8 @@ case "$1" in
     install -d "$PREFIX/$TARGET/lib"
     install -m644 libpacket.a "$PREFIX/$TARGET/lib/"
     mv wpcap/libpcap/Win32/Include/ip6_misc.h wpcap/libpcap/Win32/Include/IP6_misc.h
-    $SED 's,-DHAVE_AIRPCAP_API,,'    -i wpcap/PRJ/GNUmakefile
+    $SED 's,(char\*)tUstr +=,tUstr +=,' -i wpcap/libpcap/inet.c
+    $SED 's,-DHAVE_AIRPCAP_API,,'   -i wpcap/PRJ/GNUmakefile
     echo -e 'libwpcap.a: ${OBJS}'   >> wpcap/PRJ/GNUmakefile
     echo -e '\t${AR} rc $@ ${OBJS}' >> wpcap/PRJ/GNUmakefile
     echo -e '\t${RANLIB} $@'        >> wpcap/PRJ/GNUmakefile
@@ -995,7 +1036,7 @@ case "$1" in
     CXX="$TARGET-g++" \
     AR="$TARGET-ar" \
     RANLIB="$TARGET-ranlib" \
-    CXXFLAGS="`$PREFIX/$TARGET/bin/xml2-config --cflags`" \
+    CXXFLAGS="-ffriend-injection `$PREFIX/$TARGET/bin/xml2-config --cflags`" \
     ./configure.pl \
         --disable-shared \
         --prefix="$PREFIX/$TARGET" \
@@ -1154,7 +1195,8 @@ case "$1" in
 --download)
     cd "$DOWNLOAD"
     tar tfz "tiff-$VERSION_tiff.tar.gz" &>/dev/null ||
-    wget -c "ftp://ftp.remotesensing.org/pub/libtiff/tiff-$VERSION_tiff.tar.gz"
+    wget -t 3 -c "ftp://ftp.remotesensing.org/pub/libtiff/tiff-$VERSION_tiff.tar.gz" ||
+    wget -c "http://dl.maptools.org/dl/libtiff/tiff-$VERSION_tiff.tar.gz"
     ;;
 
 --build)
@@ -1195,7 +1237,7 @@ case "$1" in
 --download)
     cd "$DOWNLOAD"
     tar tfj "giflib-$VERSION_giflib.tar.bz2" &>/dev/null ||
-    wget -c "http://$SOURCEFORGE_MIRROR/libungif/giflib-$VERSION_giflib.tar.bz2"
+    wget -c "http://$SOURCEFORGE_MIRROR/giflib/giflib-$VERSION_giflib.tar.bz2"
     ;;
 
 --build)
@@ -1450,7 +1492,7 @@ case "$1" in
 --download)
     cd "$DOWNLOAD"
     tar tfj "gd-$VERSION_gd.tar.bz2" &>/dev/null ||
-    wget -c "http://www.libgd.org/releases/gd-$VERSION_gd.tar.bz2" ||
+    wget -t 3 -c "http://www.libgd.org/releases/gd-$VERSION_gd.tar.bz2" ||
     wget -c "http://www.libgd.org/releases/oldreleases/gd-$VERSION_gd.tar.bz2"
     ;;
 
@@ -1558,7 +1600,8 @@ case "$1" in
         --with-sdl-prefix="$PREFIX/$TARGET" \
         --disable-sdltest \
         --disable-gtk-player \
-        --disable-opengl-player
+        --disable-opengl-player \
+        CFLAGS="-ffriend-injection"
     $MAKE install bin_PROGRAMS= sbin_PROGRAMS= noinst_PROGRAMS=
     cd "$SOURCE"
     rm -rfv "smpeg-$VERSION_smpeg.orig"
@@ -1767,7 +1810,8 @@ case "$1" in
 --download)
     cd "$DOWNLOAD"
     tar tfz "proj-$VERSION_proj.tar.gz" &>/dev/null ||
-    wget -c "ftp://ftp.remotesensing.org/proj/proj-$VERSION_proj.tar.gz"
+    wget -t 3 -c "ftp://ftp.remotesensing.org/proj/proj-$VERSION_proj.tar.gz" ||
+    wget -c "http://dl.maptools.org/dl/proj/proj-$VERSION_proj.tar.gz"
     ;;
 
 --build)
@@ -1807,7 +1851,8 @@ case "$1" in
 --download)
     cd "$DOWNLOAD"
     tar tfz "libgeotiff-$VERSION_libgeotiff.tar.gz" &>/dev/null ||
-    wget -c "ftp://ftp.remotesensing.org/pub/geotiff/libgeotiff/libgeotiff-$VERSION_libgeotiff.tar.gz"
+    wget -t 3 -c "ftp://ftp.remotesensing.org/pub/geotiff/libgeotiff/libgeotiff-$VERSION_libgeotiff.tar.gz" ||
+    wget -c "http://dl.maptools.org/dl/geotiff/libgeotiff/libgeotiff-$VERSION_libgeotiff.tar.gz"
     ;;
 
 --build)
@@ -2007,7 +2052,7 @@ case "$1" in
         bin/* \
         libexec/gcc/$TARGET/*/* \
         $TARGET/bin/* || echo "strip error ignored"
-    $TARGET-strip -Sv \
+    $TARGET-strip -gv \
         lib/gcc/$TARGET/*/*.a \
         lib/gcc/$TARGET/*/*.o \
         $TARGET/lib/*.a \
